import util
import sys


class Payments:

    def __init__(self):
        return

    class Transaction:

        class Payment:

            def __init__(self, kwargs):
                auth = Payments.Transaction.Payment.Authentication(util.get_arg(kwargs, 'authentication'))
                self.authentication = auth

            class Authentication:

                def __init__(self, kwargs):
                    self.method = util.get_arg(kwargs, 'method')
                    self.url = util.get_arg(kwargs, 'url')
                    self.params = util.get_arg(kwargs, 'params')

        def __init__(self, kwargs):
            self.order_id = util.get_arg(kwargs, 'order_id')
            self.txn_id = util.get_arg(kwargs, 'txn_id')
            self.status = util.get_arg(kwargs, 'status')
            self.payment = Payments.Transaction.Payment(util.get_arg(kwargs, 'payment'))

    class PaymentMethod:

        def __init__(self, kwargs):
            self.payment_method_type = util.get_arg(kwargs, 'payment_method_type')
            self.payment_method = util.get_arg(kwargs, 'payment_method')
            self.description = util.get_arg(kwargs, 'description')

    @staticmethod
    def create_card_payment(**kwargs):
        method = 'POST'
        url = '/txns'
        parameters = {
            'payment_method_type' : 'CARD',
            'format' : 'json'
        }

        # required_args checks for presence of necessary parameters
        required_args = {}
        for key in ['order_id', 'merchant_id', 'card_token', 'card_number', 'name_on_card',
                    'card_exp_year', 'card_exp_month', 'card_security_code', 'save_to_locker', 'redirect_after_payment']:
            required_args[key] = False

        for key, value in kwargs.iteritems():
            parameters[key] = value
            required_args[key] = True

        # Either token or card number validation
        if required_args['card_token'] == False and next((True for val in ['card_number', 'name_on_card',
                    'card_exp_year','card_exp_month', 'card_security_code'] if required_args[val] == False), False):
            raise Exception('Either [card_token] or [card_number, name_on_card, card_exp_year, card_exp_month, card_security_code] are required arguments for Payments.create_card_payment()\n')

        # Warn user about missing necessary parameters
        for key in ['order_id', 'merchant_id', 'save_to_locker', 'redirect_after_payment']:
            if required_args[key] is False:
                raise Exception('%s is a required argument for Payments.create_card_payment()\n' % key)

        # Warn user about unhandled parameters
        for key in parameters:
            if key not in ['order_id', 'merchant_id', 'payment_method_type', 'payment_method', 'card_token',
                           'name_on_card',
                           'card_number', 'card_exp_year', 'card_exp_month', 'card_security_code', 'save_to_locker',
                           'redirect_after_payment', 'format']:
                sys.stderr.write('%s not a valid argument for Payments.create_card_payment()\n' % key)

        # Type checks
        for key, value in kwargs.iteritems():
            if key == 'save_to_locker':
                if type(parameters['save_to_locker']) is not bool:
                    sys.stderr.write('save_to_locker should be of type bool\n')
            if key == 'redirect_after_payment':
                if type(parameters['redirect_after_payment']) is not bool:
                    sys.stderr.write('redirect_after_payment should be of type bool\n')

        response = util.request(method, url, parameters).json()
        payment = Payments.Transaction(response)
        return payment

    @staticmethod
    def create_net_banking_payment(**kwargs):
        method = 'POST'
        url = '/txns'
        parameters = {
            'payment_method_type' : 'NB',
            'format' : 'json'
        }

        # required_args checks for presence of necessary parameters
        required_args = {}
        for key in ['order_id', 'merchant_id', 'payment_method', 'redirect_after_payment']:
            required_args[key] = False

        for key, value in kwargs.iteritems():
            parameters[key] = value
            required_args[key] = True

        # Warn user about missing necessary parameters
        for key in ['order_id', 'merchant_id', 'payment_method', 'redirect_after_payment']:
            if required_args[key] is False:
                raise Exception('%s is a required argument for Payments.create_net_banking_payment()\n' % key)

        # Warn user about unhandled parameters
        for key in parameters:
            if key not in ['order_id', 'merchant_id', 'payment_method_type', 'payment_method', 'redirect_after_payment',
                           'format']:
                sys.stderr.write('%s not a valid argument for Payments.create_net_banking_payment()\n' % key)

        response = util.request(method, url, parameters).json()
        payment = Payments.Transaction(response)
        return payment

    @staticmethod
    def create_wallet_payment(**kwargs):
        method = 'POST'
        url = '/txns'
        parameters = {
            'payment_method_type' : 'WALLET',
            'format' : 'json'
        }

        # required_args checks for presence of necessary parameters
        required_args = {}
        for key in ['order_id', 'merchant_id', 'payment_method', 'redirect_after_payment']:
            required_args[key] = False

        for key, value in kwargs.iteritems():
            parameters[key] = value
            required_args[key] = True

        # Warn user about missing necessary parameters
        for key in ['order_id', 'merchant_id', 'payment_method', 'redirect_after_payment']:
            if required_args[key] is False:
                raise Exception('%s is a required argument for Payments.create_wallet_payment()\n' % key)

        # Warn user about unhandled parameters
        for key in parameters:
            if key not in ['order_id', 'merchant_id', 'payment_method', 'payment_method_type', 'redirect_after_payment', 'format', 'direct_wallet_token']:
                sys.stderr.write('%s not a valid argument for Payments.create_wallet_payment()\n' % key)

        # Type checks
        for key, value in kwargs.iteritems():
            if key == 'redirect_after_payment':
                if type(parameters['redirect_after_payment']) is not bool:
                    sys.stderr.write('redirect_after_payment should be of type bool\n')

        response = util.request(method, url, parameters).json()
        payment = Payments.Transaction(response)
        return payment

    @staticmethod
    def get_payment_methods(**kwargs):
        merchant_id = util.get_arg(kwargs,'merchant_id')
        if merchant_id is None:
            raise util.InvalidArguementException('`merchant_id` is a required argument for Payments.get_payment_methods()\n')

        method = 'GET'
        url = '/merchants/%s/paymentmethods' % (merchant_id)

        response = util.request(method, url, {}).json()
        response = util.get_arg(response,'payment_methods')
        payment_methods = []
        if response is not None:
            for payment_method in response:
                payment_method = Payments.PaymentMethod(payment_method)
                payment_methods.append(payment_method)
        return payment_methods



import config
import requests
import juspay
from JuspayException import *


def check_param(kwargs, string):
    for key,value in kwargs.iteritems():
        if key.startswith(string):
            return True
    return False


def get_arg(kwargs, param):
    if kwargs is None:
        return None
    elif param in kwargs:
        return kwargs[param]
    else:
        return None


def request(method, url, parameters):
    try:
        if juspay.environment == 'production':
            server = 'https://api.juspay.in'
        elif juspay.environment == 'sandbox':
            server = 'https://sandbox.juspay.in'
        else:
            raise Exception("environment variable can only be 'production' or 'sandbox'")
        # Wrapper for requests
        header ={'version':config.api_version,
                 'User-Agent':'Python SDK'}
        if method.upper() == 'GET':
            response = requests.get(server + url, headers=header, params=parameters, auth=(juspay.api_key, ''))
        else:
            response = requests.post(server + url, headers=header, data=parameters, auth=(juspay.api_key, ''))

        # Report error if response is not 200 ("OK")
        if response.status_code >= 200 and response.status_code < 300:
            return response
        elif response.status_code in [400,404]:
            raise InvalidRequestException(json_body=response.content, http_status=response.status_code, request_params=parameters)
        elif response.status_code == 401:
            raise AuthenticationException(json_body=response.content, http_status=response.status_code, request_params=parameters)
        else:
            raise APIException(message="Internal server error. Please contact support@juspay.in.",
                               http_status=response.status_code,
                               request_params=parameters)
    except IOError as err:
        raise APIConnectionException(message=str(err), request_params=parameters)


'''
class list_response:

    def __init__(self, count, offset, total, list):

        self.offset = offset
        self.count = count
        self.total = total
        self.list = list
'''



import util
import sys

from JuspayException import InvalidArguementException

class Customers:

    def __init__(self):
        return

    class Customer:

        def __init__(self, kwargs):
            self.id = util.get_arg(kwargs, 'id')
            self.object = util.get_arg(kwargs, 'object')
            self.date_created = util.get_arg(kwargs, 'date_created')
            self.last_updated = util.get_arg(kwargs, 'last_updated')
            self.email_address = util.get_arg(kwargs, 'email_address')
            self.first_name = util.get_arg(kwargs, 'first_name')
            self.last_name = util.get_arg(kwargs, 'last_name')
            self.mobile_country_code = util.get_arg(kwargs, 'mobile_country_code')
            self.mobile_number = util.get_arg(kwargs, 'mobile_number')
            self.object_reference_id = util.get_arg(kwargs, 'object_reference_id')

    @staticmethod
    def create(**kwargs):
        if kwargs is None or len(kwargs) == 0:
            raise InvalidArguementException()

        method = 'POST'
        url = '/customers'
        response = util.request(method, url, kwargs).json()
        customer = Customers.Customer(response)
        return customer

    @staticmethod
    def list(**kwargs):
        method = 'GET'
        url = '/customers'
    	offset = util.get_arg(kwargs,'offset')
    	count = util.get_arg(kwargs,'count')

    	if count is None and offset is None:
            sys.stderr.write('`count` & `offset` can be passed if required.\n')

        response = util.request(method, url, kwargs or {}).json()
        response_list = util.get_arg(response,'list')
        customers_list = []
        customer_list_response = {}
        if response is not None:
            for customer in response_list:
                customer = Customers.Customer(customer)
                customers_list.append(customer)
            customer_list_response = {
                'count' : response['count'],
                'offset': response['offset'],
                'total' : response['total'],
                'list' :customers_list
            }
        return customer_list_response

    @staticmethod
    def get(**kwargs):
        id = util.get_arg(kwargs,'id')

        if id is None:
            raise InvalidArguementException(message='`id` is a required argument for Customers.list()\n')

        method = 'GET'
        url = '/customers/%s' % (id)

        response = util.request(method, url, kwargs).json()
        customer = Customers.Customer(response)
        return customer

    @staticmethod
    def update(**kwargs):
        id = util.get_arg(kwargs,'id')

        if id is None:
            raise InvalidArguementException(message='`id` is a required argument for Customers.update()\n')

        method = 'POST'
        url = '/customers/%s' % (id)

        response = util.request(method, url, kwargs).json()
        customer = Customers.Customer(response)
        return customer

    @staticmethod
    def delete(**kwargs):
        raise NotImplementedError()



import util
from JuspayException import InvalidArguementException

class Wallets:

    def __init__(self):
        return 

    class Wallet:

        def __init__(self, kwargs):
            self.id = util.get_arg(kwargs, 'id')
            self.object = util.get_arg(kwargs, 'object')
            self.wallet = util.get_arg(kwargs, 'wallet')
            self.token = util.get_arg(kwargs, 'token')
            self.linked = util.get_arg(kwargs, 'linked')
            self.current_balance = util.get_arg(kwargs, 'current_balance')
            self.last_refreshed = util.get_arg(kwargs, 'last_refreshed')

    @staticmethod
    def create(**kwargs):
        customer_id = util.get_arg(kwargs,'customer_id')
        gateway = util.get_arg(kwargs,'gateway')
        if customer_id is None or gateway is None:
            raise InvalidArguementException('`customer_id` & `gateway` are required arguments for Wallets.create()\n')

        method = 'POST'
        url = '/customers/%s/wallets' % (customer_id)

        response = util.request(method, url, kwargs).json()
        wallet = Wallets.Wallet(response)
        return wallet

    @staticmethod
    def create_and_authenticate(**kwargs):
        customer_id = util.get_arg(kwargs,'customer_id')
        gateway = util.get_arg(kwargs,'gateway')
        if customer_id is None or gateway is None:
            raise InvalidArguementException('`customer_id` & `gateway` are required arguments for '
                                            'Wallets.create_and_authenticate()\n')

        method = 'POST'
        url = '/customers/%s/wallets' % (customer_id)
        kwargs['command'] = 'authenticate'

        response = util.request(method, url, kwargs).json()
        wallet = Wallets.Wallet(response)
        return wallet

    @staticmethod
    def list(**kwargs):
        customer_id= util.get_arg(kwargs,'customer_id')
        order_id = util.get_arg(kwargs,'order_id')

        if customer_id is None and order_id is None:
            raise InvalidArguementException('`customer_id` or `order_id` is a required argument for Wallets.list()\n')

        method = 'GET'
        if customer_id is not None:
            url = '/customers/%s/wallets' % (customer_id)
        else:
            url = '/orders/%s/wallets' % (order_id)

        response = util.request(method, url, {}).json()
        response = util.get_arg(response,'list')
        wallets = []
        if response is not None:
            for wallet in response:
                wallet = Wallets.Wallet(wallet)
                wallets.append(wallet) 
        return wallets

    @staticmethod
    def refreshBalance(**kwargs):
        customer_id = util.get_arg(kwargs,'customer_id')
        if customer_id is None:
            raise InvalidArguementException('`customer_id` is a required argument for Wallets.refreshBalance()\n')

        method = 'GET'
        url = '/customers/%s/wallets/refresh-balances' % (customer_id)

        response = util.request(method, url, {}).json()
        response = util.get_arg(response,'list')
        wallets = []
        if response is not None:
            for wallet in response:
                wallet = Wallets.Wallet(wallet)
                wallets.append(wallet) 
        return wallets

    @staticmethod
    def authenticate(**kwargs):
        wallet_id = util.get_arg(kwargs,'wallet_id')
        if wallet_id is None:
            raise InvalidArguementException('`wallet_id` is a required argument for Wallets.authenticate()\n')

        method = 'POST'
        url = '/wallets/%s' % (wallet_id)
        parameters = {
            'command' : 'authenticate'
        }
        response = util.request(method, url, parameters).json()
        wallet = Wallets.Wallet(response)
        return wallet

    @staticmethod
    def refresh_by_wallet_id(**kwargs):
        wallet_id = util.get_arg(kwargs,'wallet_id')
        if wallet_id is None:
            raise InvalidArguementException('`wallet_id` is a required argument for Wallets.refresh()\n')

        method = 'GET'
        url = '/wallets/%s' % (wallet_id)
        parameters = {
            'command' : 'refresh'
        }
        response = util.request(method, url, parameters).json()
        wallet = Wallets.Wallet(response)
        return wallet

    @staticmethod
    def link(**kwargs):
        wallet_id = util.get_arg(kwargs,'wallet_id')
        otp = util.get_arg(kwargs,'otp')
        if wallet_id is None or otp is None:
            raise InvalidArguementException('`wallet_id` & `otp` are required arguments for Wallets.link()\n')

        method = 'POST'
        url = '/wallets/%s' % (wallet_id)
        kwargs['command'] = 'link'
        response = util.request(method, url, kwargs).json()
        wallet = Wallets.Wallet(response)
        return wallet

    @staticmethod
    def delink(**kwargs):
        wallet_id = util.get_arg(kwargs,'wallet_id')
        if wallet_id is None:
            raise InvalidArguementException('`wallet_id` is a required argument for Wallets.delink()\n')

        method = 'POST'
        url = '/wallets/%s' % (wallet_id)
        parameters = {
            'command' : 'delink'
        }
        response = util.request(method, url, parameters).json()
        wallet = Wallets.Wallet(response)
        return wallet



class JuspayException(Exception):

    def __init__(self, http_status=None, message=None,
                 json_body=None, headers=None, request_params= None):
        super(JuspayException, self).__init__(json_body)

        self._message = message
        self.http_status = http_status
        self.json_body = json_body
        self.headers = headers or {}
        self.request_params = request_params

    def __str__(self):
        if self._message is not None and self.http_status is not None:
            error_response = u"ERROR - Request failed with httpResponseCode : {0}.\nerrorMessage : {1}".format(self.http_status, self._message)
        elif self._message is not None:
            error_response =  u"ERROR - Request failed. Message : {0}".format(self._message)
        elif self.json_body is not None:
            error_response = u"ERROR - Request failed with httpResponseCode : {0}\n{1}".format(self.http_status, self.json_body)
        else:
            error_response = u"ERROR - Request failed with unknown reason, please contact support@juspay.in."
        if self.request_params:
            error_response = "\n{0}\nRequest Parameters : {1}".format(error_response, self.request_params)
        return error_response or {}

class InvalidRequestException(JuspayException):
    pass

class AuthenticationException(JuspayException):
    pass

class APIException(JuspayException):
    pass

class APIConnectionException(JuspayException):
    pass

class InvalidArguementException(JuspayException):
    def __init__(self, message=None):
        super(InvalidArguementException, self)\
            .__init__(message=message or "Please pass requiered parameters.")



import unittest
import datetime
import time
from Orders import Orders
import types
import juspay


def tab(text):
    count = 0
    output = ""
    text = text.split('\n')
    for line in text:
        if '}' in line:
            count -= 1
        i = 0
        while i < count:
            output += '\t'
            i += 1
        if '{' in line:
            count += 1
        output += line
        output += '\n'
    return output


def var_dump(obj, depth=4, l=""):

    non_null_dict = {}
    if depth<0: return repr(obj)
    if isinstance(obj, dict):
        objdict = obj
    else:
        canprint=lambda o:isinstance(o, (int, float, str, unicode, bool, types.NoneType, types.LambdaType))
        try:
            if canprint(obj) or sum(not canprint(o) for o in obj) == 0: return repr(obj)
        except TypeError, e:
            pass
        try:
            return "[\n" + "\n".join(l + var_dump(k, depth=depth-1, l=l+"  ") + "," for k in obj) + "\n" + l + "]"
        except TypeError, e:
            name = (hasattr(obj, '__class__') and obj.__class__.__name__ or type(obj).__name__)
            objdict = {}
            for a in dir(obj):
                if a[:2] != "__" and (not hasattr(obj, a) or not hasattr(getattr(obj, a), '__call__')):
                    if a == a.lower():
                        try: objdict[a] = getattr(obj, a)
                        except Exception, e: objdict[a] = str(e)
    name = (hasattr(obj, '__class__') and obj.__class__.__name__ or type(obj).__name__)
    for k,v in objdict.iteritems():
        if v is not None and v != '':
            non_null_dict[k]=v
    return "\n<" + name + ">" + "\n{\n" + "\n".join(l + repr(k) + ": " + var_dump(v, depth=depth-1) + "," for k, v in non_null_dict.iteritems()) + "\n" + l + "}\n"


class Test(unittest.TestCase):

    juspay.api_key = '4168A8A476B84DBCAF409C24F379BAC5'
    juspay.environment = 'production'

    def setUp(self):
        self.timestamp = int(time.mktime(datetime.datetime.timetuple(datetime.datetime.now())))

    def test__orders(self):

        # Test for list
        order_list = Orders.list()
        self.assertIsNotNone(order_list)
        for order in order_list['list']:
            self.assertIsInstance(order, Orders.Order)
        print tab(var_dump(order_list,3))

if __name__ == '__main__':
    unittest.main()



import util
import sys
from JuspayException import InvalidArguementException

class Cards:

    def __init__(self):
        return

    class Card:

        def __init__(self, kwargs):
            self.name = util.get_arg(kwargs, 'name_on_card')
            self.exp_year = util.get_arg(kwargs, 'card_exp_year')
            self.reference = util.get_arg(kwargs, 'card_reference')
            self.exp_month = util.get_arg(kwargs, 'card_exp_month')
            self.expired = util.get_arg(kwargs, 'expired')
            self.fingerprint = util.get_arg(kwargs, 'card_fingerprint')
            self.isin = util.get_arg(kwargs, 'card_isin')
            self.type = util.get_arg(kwargs, 'card_type')
            self.issuer = util.get_arg(kwargs, 'card_issuer')
            self.brand = util.get_arg(kwargs, 'card_brand')
            self.token = util.get_arg(kwargs, 'card_token')
            self.nickname = util.get_arg(kwargs, 'nickname')
            self.number = util.get_arg(kwargs, 'card_number')
            self.deleted = util.get_arg(kwargs, 'deleted')

    @staticmethod
    def create(**kwargs):
        if kwargs is None or len(kwargs) == 0:
            raise InvalidArguementException()

        method = 'POST'
        url = '/card/add'
        response = util.request(method, url, kwargs).json()
        card = Cards.Card(response)
        return card

    @staticmethod
    def list(**kwargs):
        customer_id = util.get_arg(kwargs,'customer_id')
        if customer_id is None:
            raise InvalidArguementException('`customer_id` is a required argument for Cards.list()\n')

        method = 'GET'
        url = '/card/list'
        response = util.request(method, url, kwargs).json()
        response = util.get_arg(response,'cards')
        cards = []
        if response is not None:
            for card in response:
                card = Cards.Card(card)
                cards.append(card)
        return cards

    @staticmethod
    def delete(**kwargs):
        card_token = util.get_arg(kwargs,'card_token')
        if card_token is None:
            raise InvalidArguementException('`card_token` is a required argument for Cards.delete()\n')

        method = 'POST'
        url = '/card/delete'
        response = util.request(method, url, kwargs).json()
        card = Cards.Card(response)
        return card



import util
import sys
from Cards import Cards
from JuspayException import InvalidArguementException


class Orders:

    def __init__(self):
        return

    all_input_params = ['order_id', 'amount', 'currency', 'customer_id', 'customer_email', 'customer_phone',
                        'description', 'product_id', 'gateway_id', 'return_url', 'billing_address_first_name',
                        'billing_address_last_name', 'billing_address_line1', 'billing_address_line2',
                        'billing_address_line3', 'billing_address_city', 'billing_address_state',
                        'billing_address_country', 'billing_address_postal_code', 'billing_address_phone',
                        'billing_address_country_code_iso', 'shipping_address_first_name',
                        'shipping_address_last_name', 'shipping_address_line1', 'shipping_address_line2',
                        'shipping_address_line3', 'shipping_address_city', 'shipping_address_state',
                        'shipping_address_country', 'shipping_address_postal_code', 'shipping_address_phone',
                        'shipping_address_country_code_iso', 'udf1', 'udf2', 'udf3', 'udf4', 'udf5', 'udf6', 'udf7',
                        'udf8', 'udf9', 'udf10']

    class Order:

        class Address:

            def __init__(self, address_type, kwargs):
                self.type = address_type
                self.first_name = util.get_arg(kwargs, self.type + '_address_first_name')
                self.last_name = util.get_arg(kwargs, self.type + '_address_last_name')
                self.line1 = util.get_arg(kwargs, self.type + '_address_line1')
                self.line2 = util.get_arg(kwargs, self.type + '_address_line2')
                self.line3 = util.get_arg(kwargs, self.type + '_address_line3')
                self.city = util.get_arg(kwargs, self.type + '_address_city')
                self.state = util.get_arg(kwargs, self.type + '_address_state')
                self.country = util.get_arg(kwargs, self.type + '_address_country')
                self.postal_code = util.get_arg(kwargs, self.type + '_address_postal_code')
                self.phone = util.get_arg(kwargs, self.type + '_address_phone')
                self.country_code_iso = util.get_arg(kwargs, self.type + '_address_country_code_iso')

        class Refund:

                def __init__(self, kwargs):
                    self.id = util.get_arg(kwargs, 'refund_id')
                    self.ref = util.get_arg(kwargs, 'ref')
                    self.amount = util.get_arg(kwargs, 'amount')
                    self.created = util.get_arg(kwargs, 'created')
                    self.status = util.get_arg(kwargs, 'status')

        class GatewayResponse:

                def __init__(self, kwargs):
                    self.rrn = util.get_arg(kwargs, 'rrn')
                    self.epg_txn_id = util.get_arg(kwargs, 'epg_txn_id')
                    self.auth_id_code = util.get_arg(kwargs, 'auth_id_code')
                    self.txn_id = util.get_arg(kwargs, 'txn_id')
                    self.resp_code = util.get_arg(kwargs, 'resp_code')
                    self.resp_message = util.get_arg(kwargs, 'resp_message')

        class PaymentLink:
                def __init__(self, kwargs):
                    self.web = util.get_arg(kwargs, 'web')
                    self.mobile = util.get_arg(kwargs, 'mobile')
                    self.iframe = util.get_arg(kwargs, 'iframe')

        def __init__(self, kwargs):

            if util.check_param(kwargs,'billing_address'):
                billing_address = Orders.Order.Address("billing", kwargs)
            else:
                billing_address = None
            if util.check_param(kwargs,'shipping_address'):
                shipping_address = Orders.Order.Address("shipping", kwargs)
            else:
                shipping_address = None
            if util.get_arg(kwargs, 'card') is not None:
                card = Cards.Card(util.get_arg(kwargs, 'card'))
            else:
                card = None
            if util.get_arg(kwargs, 'payment_gateway_response') is not None:
                gateway_response = Orders.Order.GatewayResponse(util.get_arg(kwargs, 'payment_gateway_response'))
            else:
                gateway_response = None
            if 'refunds' in kwargs:
                refunds = []
                for refund in kwargs['refunds']:
                    refund_obj = Orders.Order.Refund(refund)
                    refunds.append(refund_obj)
            else:
                refunds = None

            self.id = util.get_arg(kwargs, 'id')
            self.merchant_id = util.get_arg(kwargs, 'merchant_id')
            self.order_id = util.get_arg(kwargs, 'order_id')
            self.status = util.get_arg(kwargs, 'status')
            self.status_id = util.get_arg(kwargs, 'status_id')
            self.amount = util.get_arg(kwargs, 'amount')
            self.currency = util.get_arg(kwargs, 'currency')
            self.customer_id = util.get_arg(kwargs, 'customer_id')
            self.customer_email = util.get_arg(kwargs, 'customer_email')
            self.customer_phone = util.get_arg(kwargs, 'customer_phone')
            self.product_id = util.get_arg(kwargs, 'product_id')
            self.return_url = util.get_arg(kwargs, 'return_url')
            self.description = util.get_arg(kwargs, 'description')
            self.billing_address = billing_address
            self.shipping_address = shipping_address
            self.udf1 = util.get_arg(kwargs, 'udf1')
            self.udf2 = util.get_arg(kwargs, 'udf2')
            self.udf3 = util.get_arg(kwargs, 'udf3')
            self.udf4 = util.get_arg(kwargs, 'udf4')
            self.udf5 = util.get_arg(kwargs, 'udf5')
            self.udf6 = util.get_arg(kwargs, 'udf6')
            self.udf7 = util.get_arg(kwargs, 'udf7')
            self.udf8 = util.get_arg(kwargs, 'udf8')
            self.udf9 = util.get_arg(kwargs, 'udf9')
            self.udf10 = util.get_arg(kwargs, 'udf10')
            self.txn_id = util.get_arg(kwargs, 'txn_id')
            self.gateway_id = util.get_arg(kwargs, 'gateway_id')
            self.bank_error_code = util.get_arg(kwargs, 'bank_error_code')
            self.bank_error_message = util.get_arg(kwargs, 'bank_error_message')
            self.refunded = util.get_arg(kwargs, 'refunded')
            self.amount_refunded = util.get_arg(kwargs, 'amount_refunded')
            self.payment_method_type = util.get_arg(kwargs, 'payment_method_type')
            self.payment_method = util.get_arg(kwargs, 'payment_method')
            self.card = card
            self.gateway_response = gateway_response
            self.refunds = refunds
            self.payment_links = Orders.Order.PaymentLink(kwargs['payment_links'])

    @staticmethod
    def create(**kwargs):
        order = util.get_arg(kwargs,'order_id')
        if order is None:
            raise InvalidArguementException('`order_id` is a required argument for Order.create()\n')

        method = 'POST'
        url = '/order/create'
        response = util.request(method, url, kwargs).json()
        order = Orders.Order(response)
        return order

    @staticmethod
    def status(**kwargs):
        order = util.get_arg(kwargs,'order_id')
        if order is None:
            raise InvalidArguementException('`order_id` is a required argument for Order.status()\n')

        method = 'GET'
        url = '/order/status'
        response = util.request(method, url, kwargs).json()
        order = Orders.Order(response)
        return order

    @staticmethod
    def list(**kwargs):
        method = 'GET'
        url = '/order/list'

        response = util.request(method, url, kwargs or {}).json()
        returned_list = []
        for order in response['list']:
            order = Orders.Order(order)
            returned_list.append(order)
        order_list_response = {'count' : response['count'],
                               'offset': response['offset'],
                               'total' : response['total'],
                               'list' :returned_list}
        return order_list_response

    @staticmethod
    def update(**kwargs):
        order_id = util.get_arg(kwargs,'order_id')
        if order_id is None:
            raise InvalidArguementException('`order_id` is a required argument for Order.update()\n')

        method = 'POST'
        url = '/order/create'
        response = util.request(method, url, kwargs).json()
        order = Orders.Order(response)
        return order

    @staticmethod
    def refund(**kwargs):
        order_id = util.get_arg(kwargs,'order_id')
        unique_request_id = util.get_arg(kwargs, 'unique_request_id')
        if order_id is None or unique_request_id is None:
            raise InvalidArguementException('`order_id` & `unique_request_id` is a required argument for Order.refund()\n')

        method = 'POST'
        url = '/order/refund'
        response = util.request(method, url, kwargs).json()
        order = Orders.Order(response)
        return order



from setuptools import setup, find_packages

setup(
    name = 'juspay',
    version = '1.1.2',
    description = 'Python client for JusPay Express Checkout API',
    url = 'https://www.juspay.in ',
    author = 'JusPay',
    author_email = 'ninjas@juspay.in',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    packages = find_packages(),
    keywords='juspay expresscheckout payments',
    install_requires = ['requests'],
)



import unittest
import datetime
import time
from Cards import Cards
from Orders import Orders
from Payments import Payments
import types
import juspay
from pprint import pprint


def tab(text):
    count = 0
    output = ""
    text = text.split('\n')
    for line in text:
        if '}' in line:
            count -= 1
        i = 0
        while i < count:
            output += '\t'
            i += 1
        if '{' in line:
            count += 1
        output += line
        output += '\n'
    return output


def var_dump(obj, depth=4, l=""):

    non_null_dict = {}
    if depth<0: return repr(obj)
    if isinstance(obj, dict):
        objdict = obj
    else:
        canprint=lambda o:isinstance(o, (int, float, str, unicode, bool, types.NoneType, types.LambdaType))
        try:
            if canprint(obj) or sum(not canprint(o) for o in obj) == 0: return repr(obj)
        except TypeError, e:
            pass
        try:
            return "[\n" + "\n".join(l + var_dump(k, depth=depth-1, l=l+"  ") + "," for k in obj) + "\n" + l + "]"
        except TypeError, e:
            name = (hasattr(obj, '__class__') and obj.__class__.__name__ or type(obj).__name__)
            objdict = {}
            for a in dir(obj):
                if a[:2] != "__" and (not hasattr(obj, a) or not hasattr(getattr(obj, a), '__call__')):
                    if a == a.lower():
                        try: objdict[a] = getattr(obj, a)
                        except Exception, e: objdict[a] = str(e)
    name = (hasattr(obj, '__class__') and obj.__class__.__name__ or type(obj).__name__)
    for k,v in objdict.iteritems():
        if v is not None and v != '':
            non_null_dict[k]=v
    return "\n<" + name + ">" + "\n{\n" + "\n".join(l + repr(k) + ": " + var_dump(v, depth=depth-1) + "," for k, v in non_null_dict.iteritems()) + "\n" + l + "}\n"


class Test(unittest.TestCase):

    juspay.api_key = '4168A8A476B84DBCAF409C24F379BAC5'
    juspay.environment = 'production'

    def setUp(self):
        self.timestamp = int(time.mktime(datetime.datetime.timetuple(datetime.datetime.now())))

        self.order1 = Orders.create(order_id=self.timestamp, amount=1000)
        self.assertIsInstance(self.order1, Orders.Order)
        self.assertEqual(self.order1.status, "CREATED")
        self.order2 = Orders.create(order_id=self.timestamp + 1, amount=1000)

    def test__orders(self):

        # Test for get_status
        status = Orders.status(order_id=self.timestamp)
        self.assertIsInstance(status, Orders.Order)
        self.assertEqual(float(status.order_id), self.timestamp)
        #print tab(var_dump(status, 1))

        # Test for list
        order_list = Orders.list()
        self.assertIsNotNone(order_list)
        for order in order_list['list']:
            self.assertIsInstance(order, Orders.Order)
        #print tab(var_dump(order_list,2))

        # Test for update
        updated_order = Orders.update(order_id=self.timestamp, amount=500)
        status = Orders.status(order_id=self.timestamp)
        self.assertEqual(status.amount, updated_order.amount)
        #print tab(var_dump(updated_order))

        # Test for payment links
        self.assertIsNotNone(order.payment_links)


    def test__refund(self):
        # Test for refund
        refunded_order = Orders.refund(unique_request_id=self.timestamp,order_id=1465833326,amount=10)
        self.assertIsNotNone(refunded_order)
        #print tab(var_dump(refunded_order))

    @staticmethod
    def delete_all_cards():
        card_list = Cards.list(customer_id='user')
        for card in card_list:
            Cards.delete(card_token=card.token)

    def test__cards(self):

        # Test for add
        card = Cards.create(merchant_id='shreyas', customer_id="user", customer_email='abc@xyz.com',
                                card_number=str(int(self.timestamp)*(10**6)), card_exp_year='20', card_exp_month='12')
        self.assertIsNotNone(card.reference)
        self.assertIsNotNone(card.token)
        self.assertIsNotNone(card.fingerprint)
        #print tab(var_dump(card))

        # Test for delete
        deleted_card = Cards.delete(card_token=card.token)
        self.assertTrue(deleted_card.deleted)
        #print tab(var_dump(deleted_card))

        # Test for list
        Test.delete_all_cards()
        Cards.create(merchant_id='shreyas', customer_id="user", customer_email='abc@xyz.com',
                         card_number=str(int(self.timestamp) * (10 ** 6)), card_exp_year='20', card_exp_month='12')
        Cards.create(merchant_id='shreyas', customer_id="user", customer_email='abc@xyz.com',
                         card_number=str((int(self.timestamp)+1) * (10 ** 6)), card_exp_year='20', card_exp_month='12')
        card_list = Cards.list(customer_id='user')
        self.assertIsNotNone(card_list)
        self.assertEqual(len(card_list), 2)
        #print tab(var_dump(card_list))
        Test.delete_all_cards()

    def test__payments(self):

        # Test for create_card_payment
        payment = Payments.create_card_payment(order_id=1465893617,
                                                      merchant_id='juspay_recharge',
                                                      payment_method_type='CARD',
                                                      card_token='68d6b0c6-6e77-473f-a05c-b460ef983fd8',
                                                      redirect_after_payment=False,
                                                      format='json',
                                                      card_number='5243681100075285',
                                                      name_on_card='Customer',
                                                      card_exp_year='20',
                                                      card_exp_month='12', card_security_code='123',
                                                      save_to_locker=False)
        self.assertIsNotNone(payment.txn_id)
        self.assertEqual(payment.status, 'PENDING_VBV')
        #print tab(var_dump(payment))

        # Test for create_net_banking_payment
        payment = Payments.create_net_banking_payment(order_id=1465893617,
                                                             merchant_id='juspay_recharge',
                                                             payment_method_type='NB',
                                                             payment_method='NB_ICICI',
                                                             redirect_after_payment=False,
                                                             format='json')
        self.assertIsNotNone(payment.txn_id)
        self.assertEqual(payment.status, 'PENDING_VBV')
        #print tab(var_dump(payment))


if __name__ == '__main__':
    unittest.main()
